import argparse
import concurrent.futures
import glob
import logging
import os
import zipfile
from tempfile import TemporaryDirectory

import boto3
import requests

logger = logging.getLogger(__name__)


def upload_files_to_s3(file_name: str, bucket_name: str) -> str:
    s3_client = boto3.client('s3')
    s3_client.upload_file(file_name, bucket_name, "/".join(file_name.split("/")[3:]))
    return file_name


def main():
    parser = argparse.ArgumentParser(description='Upload files to S3 with concurrency')
    parser.add_argument('url', help='URL to the ZIP archive')
    parser.add_argument('bucket_name', help='S3 bucket name')
    parser.add_argument('s3_key_prefix', help='S3 key prefix for uploaded files')
    parser.add_argument('--concurrency', type=int, default=8, help='Concurrency level')
    parser.add_argument('--verbose', action='store_true', help='Enable verbose output')
    args = parser.parse_args()

    # Download the ZIP file from the provided URL
    response = requests.get(args.url)
    response.raise_for_status()
    with open("downloaded_zip_tmp.zip", 'wb') as zip_file:
        zip_file.write(response.content)

    file_names = []
    with TemporaryDirectory() as temp_dir:
        with zipfile.ZipFile("downloaded_zip_tmp.zip", 'r') as zip_ref:
            zip_ref.extractall(temp_dir)

            for filename in glob.iglob(f"{temp_dir}/**", recursive=True):
                if os.path.isfile(filename):
                    file_names.append(filename)

            with concurrent.futures.ThreadPoolExecutor(max_workers=8) as executor:
                results = [executor.submit(upload_files_to_s3, file_name=file_name, bucket_name=args.bucket_name) for file_name in file_names]
                for result in concurrent.futures.as_completed(results):
                    if args.verbose:
                        logger.info(result.result())


if __name__ == '__main__':
    main()
